@@ -145,7 +145,7 @@ class BaseSubmitter(object):
                 'QONDOR_BATCHMODE' : '1',
                 'CONDOR_CLUSTER_NUMBER' : '$(Cluster)',
                 'CONDOR_PROCESS_ID' : '$(Process)',
-                'CLUSTER_SUBMISSION_TIMESTAMP' : strftime('%Y%m%d_%H%M%S'),
+                'CLUSTER_SUBMISSION_TIMESTAMP' : strftime(qondor.TIMESTAMP_FMT),
                 },
             }
         # Try to set some more things
@@ -167,29 +167,22 @@ class BaseSubmitter(object):
             # No user specified, no big deal
             pass
         # Special settings for CMS Connect
-        if os.uname()[1] == 'login.uscms.org':
-            sites = [
-                'T3_BY_NCPHEP', 'T2_IT_Bari', 'T3_US_Baylor', 'T2_CN_Beijing', 'T3_IT_Bologna', 'T2_UK_SGrid_Bristol',
-                'T2_UK_London_Brunel', 'T0_CH_CERN', 'T2_CH_CERN', 'T2_CH_CERN_AI', 'T2_CH_CERN_HLT', 'T2_ES_CIEMAT',
-                'T1_IT_CNAF', 'T3_CN_PKU', 'T2_TH_CUNSTDA', 'T2_US_Caltech', 'T3_US_Colorado', 'T3_US_Cornell', 'T3_RU_FIAN',
-                'T3_US_FIT', 'T1_US_FNAL', 'T3_US_FNALLPC', 'T3_US_Omaha', 'T2_FR_GRIF_IRFU', 'T2_FR_GRIF_LLR', 'T2_BR_UERJ',
-                'T2_FI_HIP', 'T2_AT_Vienna', 'T2_HU_Budapest', 'T2_ES_IFCA', 'T2_RU_IHEP', 'T2_BE_IIHE', 'T3_FR_IPNL', 'T2_RU_INR',
-                'T2_FR_IPHC', 'T2_RU_ITEP', 'T2_GR_Ioannina', 'T3_US_JHU', 'T2_RU_JINR', 'T1_RU_JINR', 'T2_UA_KIPT', 'T1_DE_KIT',
-                'T2_KR_KNU', 'T3_KR_KNU', 'T3_US_Kansas', 'T2_IT_Legnaro', 'T2_BE_UCL', 'T3_RU_MEPhI', 'T2_TR_METU', 'T2_US_MIT',
-                'T2_PT_NCG_Lisbon', 'T2_PK_NCP', 'T3_TW_NCU', 'T3_TW_NTU_HEP', 'T3_US_NotreDame', 'T2_TW_NCHC', 'T2_US_Nebraska',
-                'T3_US_NU', 'T3_US_OSU', 'T3_ES_Oviedo', 'T3_UK_SGrid_Oxford', 'T1_ES_PIC', 'T2_RU_PNPI', 'T3_CH_PSI',
-                'T3_IN_PUHEP', 'T3_IT_Perugia', 'T3_US_Princeton_ICSE', 'T2_US_Purdue', 'T3_UK_London_QMUL', 'T1_UK_RAL',
-                'T3_UK_London_RHUL', 'T2_DE_RWTH', 'T3_US_Rice', 'T2_IT_Rome', 'T3_US_Rutgers', 'T2_UK_SGrid_RALPP', 'T2_RU_SINP',
-                'T2_BR_SPRACE', 'T2_CH_CSCS_HPC', 'T2_PL_Swierk', 'T3_US_MIT', 'T3_US_NERSC', 'T3_US_SDSC', 'T3_CH_CERN_CAF',
-                'T3_US_FIU', 'T3_US_FSU', 'T3_US_OSG', 'T3_US_TAMU', 'T2_IN_TIFR', 'T3_IN_TIFRCloud', 'T3_US_TTU', 'T3_IT_Trieste',
-                'T3_US_UCD', 'T3_US_UCSB', 'T2_US_UCSD', 'T3_UK_ScotGrid_GLA', 'T3_US_UMD', 'T3_US_UMiss', 'T3_CO_Uniandes',
-                'T3_KR_UOS', 'T2_MY_UPM_BIRUNI', 'T3_US_PuertoRico', 'T3_BG_UNI_SOFIA', 'T2_US_Vanderbilt', 'T2_US_Wisconsin',
-                'T3_MX_Cinvestav', 'T3_US_FNALLPC'
-                ]
+        # Maybe just .endwith('.uscms.org') ? Not sure if that would break other sites
+        if os.uname()[1] == 'login.uscms.org' or os.uname()[1] == 'login-el7.uscms.org':
+            logger.warning('Detected CMS Connect; loading specific settings')
+            # Read the central config for cmsconnect
+            try:
+                from configparser import RawConfigParser # python 3
+            except ImportError:
+                import ConfigParser # python 2
+                RawConfigParser = ConfigParser.RawConfigParser
+            cfg = RawConfigParser()
+            cfg.read('/etc/ciconnect/config.ini')
+            default_sites = cfg.get('submit', 'DefaultSites')
+            sub['+DESIRED_Sites'] = '"' + default_sites + '"'
             sub['+ConnectWrapper'] = '"2.0"'
             logger.warning('FIXME: CMS Connect settings currently hard-coded for a FNAL user')
             sub['+CMSGroups'] = '"/cms,T3_US_FNALLPC"'
-            sub['+DESIRED_Sites'] = '"' + ','.join(sites) + '"'
             sub['+MaxWallTimeMins'] = '500'
             sub['+ProjectName'] = '"cms.org.fnal"'
             sub['+SubmitFile'] = '"irrelevant.jdl"'
@@ -222,22 +215,33 @@ class BaseSubmitter(object):
         transfer_files = self.transfer_files + [f for f in preprocessor.files.values() if not seutils.has_protocol(f)]
         if len(transfer_files) > 0:
             sub['transfer_input_files'] = ','.join(transfer_files)
-        # Simple case with no items; just use the njobs variable        
-        if not preprocessor.items:
+        # Otherwise continue processing items
+        if preprocessor.items and preprocessor.rootfile_chunks:
+            raise ValueError(
+                'Both regular items and rootfiles-split-by-entries are specified; '
+                'This is not supported.'
+                )
+        elif preprocessor.items:
+            logger.info('Items:\n%s', pprint.pformat(preprocessor.items))
+            if not self.dry:
+                return htcondor_submit(sub, 1, submission_dir=self.rundir, items=preprocessor.items)
+        elif preprocessor.rootfile_chunks:
+            logger.info('Items as rootfile chunks:\n%s', pprint.pformat(preprocessor.rootfile_chunks))
+            if not self.dry:
+                return htcondor_submit(
+                    sub, 1, submission_dir=self.rundir, rootfile_chunks=preprocessor.rootfile_chunks
+                    )
+        else:
+            # Simple case with no items; just use the njobs variable with a default of 1
             njobs = int(preprocessor.variables.get('njobs', 1))
             logger.info('Submitting %s jobs with:\n%s', njobs, pprint.pformat(sub))
             if not self.dry:
                 return htcondor_submit(sub, njobs, submission_dir=self.rundir)
-        # Otherwise continue processing items
-        logger.info('Items:\n%s', pprint.pformat(preprocessor.items))
-        if not self.dry:
-            subcopy = sub.copy()
-            return htcondor_submit(subcopy, 1, submission_dir=self.rundir, items=preprocessor.items)
 
     def make_rundir(self):
         self.rundir = osp.abspath('qondor_{0}_{1}'.format(
             self.python_name,
-            strftime('%Y%m%d_%H%M%S')
+            strftime(qondor.TIMESTAMP_FMT)
             ))
         qondor.utils.create_directory(
             self.rundir,
@@ -253,6 +257,12 @@ class BaseSubmitter(object):
         qondor.Preprocessor.dump_ls_cache(cache_file)
         self.transfer_files.append(cache_file)
 
+    def dump_rootcache(self):
+        if seutils.root.USE_CACHE:
+            rootcache_file = osp.join(self.rundir, 'rootcache.tar.gz')
+            seutils.root.cache_to_file(rootcache_file)
+            self.transfer_files.append(rootcache_file)
+
     def submit(self, python_script_args=None):
         """
         Main submission method
@@ -264,6 +274,7 @@ class BaseSubmitter(object):
             self.make_rundir()
             self.copy_python_file()
             self.dump_ls_cache_file()
+            self.dump_rootcache()
             # Loop over all 'sets'
             # If there are no subsets, this is just a len(1) iterator of the preprocessing
             for i_set, preprocessor in enumerate(self.preprocessing.sets()):
@@ -355,22 +366,29 @@ class CodeSubmitter(BaseSubmitter):
             os.remove(python_file)
 
 
-def htcondor_submit(sub, njobs=1, submission_dir='.', items=None):
+def htcondor_submit(sub, njobs=1, submission_dir='.', items=None, rootfile_chunks=None):
     """
     Submits the submission dict `sub` to the best scheduler.
+    If items are passed, it submits 1 job per item, and makes sure
+    to set the item in the environment of the job in $QONDORITEM.
+    If rootfile_chunks are passed, it submits 1 job per chunk, and
+    makes sure to set the chunk in the environment of the job in
+    $QONDORROOTFILECHUNK.
+    If neither of those are set, it submits `njobs` without setting
+    anything specific per job in the job's environment.
     Returns the cluster id and class ad of the submitted job
     """
     import htcondor
     schedd = qondor.get_best_schedd(renew=True)
-    # Create a copy to keep original dict unmodified
-    # Make the transaction
     with qondor.utils.switchdir(submission_dir):
+        # Create a copy to keep original dict unmodified
+        # Make the transaction
         subcopy = sub.copy()
         with schedd.transaction() as transaction:
+            ads = []
             if items:
                 # Items logic: Turn any potential list into a ','-separated string,
                 # and set the environment variable QONDORITEM to that string.
-                ads = []
                 subcopy['environment']['QONDORITEM'] = 'placeholder' # Placeholder
                 subcopy['environment'] = htcondor_format_environment(subcopy['environment'])
                 submit_object = htcondor.Submit(subcopy)
@@ -394,10 +412,24 @@ def htcondor_submit(sub, njobs=1, submission_dir='.', items=None):
                     cluster_id = submit_object.queue(transaction, njobs, ad)
                     cluster_id = int(cluster_id)
                     ads.extend(ad)
+            elif rootfile_chunks:
+                # Set the chunk as a string in the environment as follows:
+                # rootfile,first,last,is_whole_file;rootfile,first,last,is_whole_file;...
+                format_chunk = lambda chunk: ';'.join(
+                    [','.join([str(i) for i in elements]) for elements in chunk]
+                    )
+                subcopy['environment']['QONDORROOTFILECHUNK'] = 'placeholder'
+                subcopy['environment'] = htcondor_format_environment(subcopy['environment'])
+                submit_object = htcondor.Submit(subcopy)                
+                for chunk in rootfile_chunks:
+                    change_submitobject_env_variable(submit_object, 'QONDORROOTFILECHUNK', format_chunk(chunk))
+                    ad = []
+                    cluster_id = submit_object.queue(transaction, njobs, ad)
+                    cluster_id = int(cluster_id)
+                    ads.extend(ad)
             else:
                 subcopy['environment'] = htcondor_format_environment(subcopy['environment'])
                 submit_object = htcondor.Submit(subcopy)
-                ads = []
                 cluster_id = submit_object.queue(transaction, njobs, ads)
                 cluster_id = int(cluster_id)
     logger.info('Submitted %s jobs to cluster %s', len(ads), cluster_id)
